// hub4com実行用GUIプログラム
// 3つのCOMポートを接続・切断する
// ボーレート設定機能追加
//
// 2017/10/23 伊藤大智

import controlP5.*;
import java.io.*;
import java.util.*;

ControlP5 cp5;

String textValue = "";
String[] tokens;
color clr;
int myCurrentIndex = 0;

void setup() {
  size(240,420);
  
  PFont font = createFont("arial",20);
  
  cp5 = new ControlP5(this);
  
  cp5.addTextfield("COM A") // 1番目のCOM入力
     .setPosition(20,40)
     .setSize(200,40)
     .setFont(font)
     .setFocus(true)
     .setAutoClear(false)
     ;
                 
  cp5.addTextfield("COM B") // 2番目のCOM入力
     .setPosition(20,120)
     .setSize(200,40)
     .setFont(font)
     .setAutoClear(false)
     ;
  
  cp5.addTextfield("COM C")  // 3番目のCOM入力
     .setPosition(20,200)
     .setSize(200,40)
     .setFont(font)
     .setAutoClear(false)
     ;
            
  cp5.addBang("connect") // 接続ボタン
     .setPosition(20,340)
     .setSize(60,40)
     .getCaptionLabel().align(ControlP5.CENTER, ControlP5.CENTER)
     ;    
     
  cp5.addBang("disconnect") // 接続解除ボタン
     .setPosition(90,340)
     .setSize(60,40)
     .getCaptionLabel().align(ControlP5.CENTER, ControlP5.CENTER)
     ; 
     
  cp5.addBang("clear") // クリアボタン
     .setPosition(160,340)
     .setSize(60,40)
     .getCaptionLabel().align(ControlP5.CENTER, ControlP5.CENTER)
     ; 
  
  // ボーレート設定
  tokens = new String[10];
  tokens[0] = "9600";
  tokens[1] = "19200";
  tokens[2] = "38400";
  tokens[3] = "57600";
  tokens[4] = "115200";

  List l = Arrays.asList(tokens[0], tokens[1], tokens[2],tokens[3], tokens[4]);
     
  cp5.addScrollableList("baudrate") // ボーレート設定リスト
     .setPosition(20, 280)
     .setSize(200, 120)
     .setBarHeight(20)
     .setItemHeight(20)
     .addItems(l);
     
  textFont(font);
}


void draw() {
  background(clr);
  fill(255);
  text("hub4com GUI",60,20);
  text("Baudrate [bps]",20,325);
  text(textValue, 20,410);
}


// ボーレート選択がされたとき
void baudrate(int n) {
 // println(n);  
  myCurrentIndex = n;  
}


// 接続ボタンが押されたとき
public void connect() {
  String cmdstr,strcoma,strcomb,strcomc;
  int ans;
  
  strcoma=cp5.get(Textfield.class,"COM A").getText();
  strcomb=cp5.get(Textfield.class,"COM B").getText();
  strcomc=cp5.get(Textfield.class,"COM C").getText();
  String[] location=loadStrings("hub4com_location.txt"); // hub4comの保存場所を記入
  StringBuilder buf = new StringBuilder();
  buf.append("cd ");
  buf.append(location[0]);
  buf.append(" & hub4com --route=All:All --baud=");
  buf.append(tokens[myCurrentIndex]);
  buf.append(" \\\\.\\");
  buf.append(strcoma);
  buf.append(" \\\\.\\");
  buf.append(strcomb);
  buf.append(" \\\\.\\");
  buf.append(strcomc);
  cmdstr=buf.toString();
  ans=cmd_send(cmdstr);
  System.out.println(ans);
  
  if(ans==0){
    textValue="Connected";
    clr=color(228,155,15); // 黄金色
  }
  else{
    textValue="Connection failure";
  }
}


// 接続解除が押されたとき
public void disconnect() {
  String cmdstr;
  int ans;
  cmdstr="taskkill /F /IM hub4com.exe";
  ans=cmd_send(cmdstr);
  System.out.println(ans);
  
  if(ans==0){
    textValue="Disconnected";
    clr=color(0,0,0);
  }
  else{
    textValue="Disconnection failure";
  }
  
}


// クリアボタンが押されたとき
public void clear() {
  cp5.get(Textfield.class,"COM A").clear();
  cp5.get(Textfield.class,"COM B").clear();
  cp5.get(Textfield.class,"COM C").clear();
}



// コマンドプロンプトでコマンドを実行する関数
int cmd_send(String cmdstr){
  int ans=0;
  try{
    System.out.println(cmdstr);
    ProcessBuilder pb = new ProcessBuilder("cmd.exe","/c",cmdstr);
    Process process = pb.start();
    
    // 結果を標準出力に書き出す
    System.out.println("↓　プロセス実行結果");
    System.out.println("==================================================================");
    BufferedReader in = new BufferedReader(new InputStreamReader(process.getInputStream()));
    String str;
    int i=0;
    while ((str = in.readLine()) != null) {
      System.out.println(str);
      i++;
      if(i>11){break;}
    }    
    System.out.println("==================================================================");
      
      // プロセスのexit値を取得・表示
      System.out.println("プロセスの終了値 : " + process.exitValue());
      
      // 終了処理作業
      process.destroy();
      in.close();      
      ans=process.exitValue();
      
      }catch (Exception e) {
     println(e);
  }
  return(ans);
}